@@ -1,20 +1,23 @@
 package Text::Table::Span;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2021-02-19'; # DATE
+our $DATE = '2021-02-20'; # DATE
 our $DIST = 'Text-Table-Span'; # DIST
-our $VERSION = '0.007'; # VERSION
+our $VERSION = '0.008'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
 
 use List::AllUtils qw(first firstidx max);
-use String::Pad qw(pad);
 
 use Exporter qw(import);
 our @EXPORT_OK = qw/ generate_table /;
 
+our $_split_lines_func;
+our $_pad_func;
+our $_length_height_func;
+
 # consts
 sub IDX_EXPTABLE_CELL_ROWSPAN()         {0} # number of rowspan, only defined for the rowspan head
 sub IDX_EXPTABLE_CELL_COLSPAN()         {1} # number of colspan, only defined for the colspan head
@@ -48,6 +51,28 @@ sub _divide_int_to_n_ints {
     @ints;
 }
 
+sub _vpad {
+    my ($lines, $num_lines, $width, $which) = @_;
+    return $lines if @$lines >= $num_lines; # we don't do truncate
+    my @vpadded_lines;
+    my $pad_line = " " x $width;
+    if ($which =~ /^b/) { # bottom padding
+        push @vpadded_lines, @$lines;
+        push @vpadded_lines, $pad_line for @$lines+1 .. $num_lines;
+    } elsif ($which =~ /^t/) { # top padding
+        push @vpadded_lines, $pad_line for @$lines+1 .. $num_lines;
+        push @vpadded_lines, @$lines;
+    } else { # center padding
+        my $p  = $num_lines - @$lines;
+        my $p1 = int($p/2);
+        my $p2 = $p - $p1;
+        push @vpadded_lines, $pad_line for 1..$p1;
+        push @vpadded_lines, @$lines;
+        push @vpadded_lines, $pad_line for 1..$p2;
+    }
+    \@vpadded_lines;
+}
+
 sub _get_attr {
     my ($attr_name, $y, $x, $cell_value, $table_args) = @_;
 
@@ -107,10 +132,12 @@ sub _get_exptable_cell_lines {
         $bottom_borders, $intercol_width, $y, $x) = @_;
 
     my $exptable_cell = $exptable->[$y][$x];
-    my $cell  = $exptable_cell->[IDX_EXPTABLE_CELL_ORIG];
-    my $text  = ref $cell eq 'HASH' ? $cell->{text} : $cell;
-    my $align = _get_attr('align', $y, $x, $cell, $table_args) // 'left';
-    my $pad   = $align eq 'left' ? 'r' : $align eq 'right' ? 'l' : 'c';
+    my $cell   = $exptable_cell->[IDX_EXPTABLE_CELL_ORIG];
+    my $text   = ref $cell eq 'HASH' ? $cell->{text} : $cell;
+    my $align  = _get_attr('align', $y, $x, $cell, $table_args) // 'left';
+    my $valign = _get_attr('valign', $y, $x, $cell, $table_args) // 'top';
+    my $pad    = $align eq 'left' ? 'r' : $align eq 'right' ? 'l' : 'c';
+    my $vpad   = $valign eq 'top' ? 'b' : $valign eq 'bottom' ? 't' : 'c';
     my $height = 0;
     my $width  = 0;
     for my $ic (1..$exptable_cell->[IDX_EXPTABLE_CELL_COLSPAN]) {
@@ -122,15 +149,9 @@ sub _get_exptable_cell_lines {
         $height++ if $bottom_borders->[$y+$ir-2] && $ir > 1;
     }
 
-    my @lines;
-    my @datalines = split /\R/, $text;
-    for (1..@datalines) {
-        push @lines, pad($datalines[$_-1], $width, $pad, ' ', 'truncate');
-    }
-    for (@datalines+1 .. $height) {
-        push @lines, " " x $width;
-    }
-    \@lines;
+    my @datalines = map { $_pad_func->($_, $width, $pad, ' ', 'truncate') }
+        ($_split_lines_func->($text));
+    _vpad(\@datalines, $height, $width, $vpad);
 }
 
 sub generate_table {
@@ -144,6 +165,43 @@ sub generate_table {
 
     my $bs_obj = Module::Load::Util::instantiate_class_with_optional_args({ns_prefix=>"BorderStyle"}, $bs_name);
 
+  DETERMINE_CODES: {
+        my $color = $args{color};
+        my $wide_char = $args{wide_char};
+
+        # split_lines
+        if ($color) {
+            require Text::ANSI::Util;
+            $_split_lines_func = sub { Text::ANSI::Util::ta_add_color_resets(split /\R/, $_[0]) };
+        } else {
+            $_split_lines_func = sub { split /\R/, $_[0] };
+        }
+
+        # pad & length_height
+        if ($color) {
+            if ($wide_char) {
+                require Text::ANSI::WideUtil;
+                $_pad_func           = \&Text::ANSI::WideUtil::ta_mbpad;
+                $_length_height_func = \&Text::ANSI::WideUtil::ta_mbswidth_height;
+            } else {
+                require Text::ANSI::Util;
+                $_pad_func           = \&Text::ANSI::Util::ta_pad;
+                $_length_height_func = \&Text::ANSI::Util::ta_length_height;
+            }
+        } else {
+            if ($wide_char) {
+                require Text::WideChar::Util;
+                $_pad_func           = \&Text::WideChar::Util::mbpad;
+                $_length_height_func = \&Text::WideChar::Util::mbswidth_height;
+            } else {
+                require String::Pad;
+                require Text::NonWideChar::Util;
+                $_pad_func           = \&String::Pad::pad;
+                $_length_height_func = \&Text::NonWideChar::Util::length_height;
+            }
+        }
+    }
+
     # XXX when we allow cell attrs right_border and left_border, this will
     # become array too like $exptable_bottom_borders.
     my $intercol_width = length(" " . $bs_obj->get_border_char(3, 1) . " ");
@@ -244,7 +302,7 @@ sub generate_table {
                 my $colspan = $exptable_cell->[IDX_EXPTABLE_CELL_COLSPAN];
                 my $cell = $exptable_cell->[IDX_EXPTABLE_CELL_ORIG];
                 my $text = ref $cell eq 'HASH' ? $cell->{text} : $cell;
-                my $lh = Text::NonWideChar::Util::length_height($text);
+                my $lh = $_length_height_func->($text);
                 #use DDC; say "D:length_height[$exptable_rownum,$exptable_colnum] = (".DDC::dump($text)."): ".DDC::dump($lh);
                 my $tot_intercol_widths = ($colspan-1) * $intercol_width;
                 my $tot_interrow_heights = 0; for (1..$rowspan-1) { $tot_interrow_heights++ if $exptable_bottom_borders->[$exptable_rownum+$_-1] }
@@ -500,7 +558,7 @@ Text::Table::Span - Text::Table::Tiny + support for column/row spans
 
 =head1 VERSION
 
-This document describes version 0.007 of Text::Table::Span (from Perl distribution Text-Table-Span), released on 2021-02-19.
+This document describes version 0.008 of Text::Table::Span (from Perl distribution Text-Table-Span), released on 2021-02-20.
 
 =head1 SYNOPSIS
 
@@ -685,6 +743,12 @@ String. Value is either C<"left">, C<"middle">, C<"right">. Specify text
 alignment of cells. Override table argument, but is overridden by per-column or
 per-cell attribute of the same name.
 
+=head2 valign
+
+String. Value is either C<"top">, C<"middle">, C<"bottom">. Specify vertical
+text alignment of cells. Override table argument, but is overridden by
+per-column or per-cell attribute of the same name.
+
 =head2 bottom_border
 
 Boolean.
@@ -701,6 +765,12 @@ String. Value is either C<"left">, C<"middle">, C<"right">. Specify text
 alignment of cells. Override table argument and per-row attribute of the same
 name, but is overridden by per-cell attribute of the same name.
 
+=head2 valign
+
+String. Value is either C<"top">, C<"middle">, C<"bottom">. Specify vertical
+text alignment of cells. Override table argument and per-row attribute of the
+same name, but is overridden by per-cell attribute of the same name.
+
 =head1 PER-CELL ATTRIBUTES
 
 =head2 align
@@ -708,6 +778,12 @@ name, but is overridden by per-cell attribute of the same name.
 String. Value is either C<"left">, C<"middle">, C<"right">. Override table
 argument, per-row attribute, and per-column attribute of the same name.
 
+=head2 valign
+
+String. Value is either C<"top">, C<"middle">, C<"bottom">. Specify vertical
+text alignment of cells. Override table argument, per-row attribute, and
+per-column attribute of the same name.
+
 =head2 colspan
 
 Positive integer. Default 1.
@@ -757,9 +833,15 @@ L<App::BorderStyleUtils>.
 
 =item * align
 
-String. Value is either C<"left">, C<"middle">, C<"right">. Specify text
-alignment of cells. Overriden by overridden by per-row, per-column, or per-cell
-attribute of the same name.
+String. Value is either C<"left">, C<"middle">, C<"right">. Specify horizontal
+text alignment of cells. Overriden by overridden by per-row, per-column, or
+per-cell attribute of the same name.
+
+=item * valign
+
+String. Value is either C<"top">, C<"middle">, C<"bottom">. Specify vertical
+text alignment of cells. Overriden by overridden by per-row, per-column, or
+per-cell attribute of the same name.
 
 =item * row_attrs
 
@@ -789,6 +871,18 @@ Boolean. Optional. Default 0. If set to true, will add a separator between data
 rows. Equivalent to setting C<bottom_border> or C<top_border> attribute to true
 for each row.
 
+=item * wide_char
+
+Boolean. Optional. Default false. Turn on wide character support. Cells that
+contain wide Unicode characters will still be properly aligned. Note that this
+requires optional prereq L<Text::WideChar::Util> or L<Text::ANSI::WideUtil>.
+
+=item * color
+
+Boolean. Optional. Default false. Turn on color support. Cells that contain ANSI
+color codes will still be properly aligned. Note that this requires optional
+prereq L<Text::ANSI::Util> or L<Text::ANSI::WideUtil>.
+
 =back
 
 =head1 HOMEPAGE
